@@ -3,8 +3,9 @@
 *************************************************/
 
 /* Copyright (c) University of Cambridge 1995 - 2018 */
-/* Copyright (c) The Exim Maintainers 2021 */
+/* Copyright (c) The Exim Maintainers 2023 */
 /* See the file NOTICE for conditions of use and distribution. */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 
 #include "em_hdr.h"
@@ -824,7 +825,7 @@ while (p > 0 && s[p+11] == ' ')
 
 /* Now pointing at first character of a main line. */
 
-Ustrncpy(message_id, s+p+11, MESSAGE_ID_LENGTH);
+Ustrncpy(message_id, s+p+11, MESSAGE_ID_LENGTH);	/*III*/
 message_id[MESSAGE_ID_LENGTH] = 0;
 
 /* Highlight the line being menued, and save its parameters so that it
