@@ -2,9 +2,10 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2022 */
+/* Copyright (c) The Exim Maintainers 2020 - 2024 */
 /* Copyright (c) Jeremy Harris 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* Create a static data structure with the predefined macros, to be
 included in the main Exim build */
@@ -118,6 +119,12 @@ due to conflicts with other common macros. */
 #ifdef SUPPORT_PAM
   builtin_macro_create(US"_HAVE_PAM");
 #endif
+#ifdef RADIUS_CONFIG_FILE
+  builtin_macro_create(US"_HAVE_RADIUS");
+#endif
+#ifdef CYRUS_PWCHECK_SOCKET
+  builtin_macro_create(US"_HAVE_PWCHECK");
+#endif
 #ifdef EXIM_PERL
   builtin_macro_create(US"_HAVE_PERL");
 #endif
@@ -153,6 +160,9 @@ due to conflicts with other common macros. */
 #ifndef DISABLE_DNSSEC
   builtin_macro_create(US"_HAVE_DNSSEC");
 #endif
+#ifndef DISABLE_ESMTP_LIMITS
+  builtin_macro_create(US"_HAVE_ESMTP_LIMITS");
+#endif
 #ifndef DISABLE_EVENT
   builtin_macro_create(US"_HAVE_EVENT");
 #endif
@@ -204,6 +214,24 @@ due to conflicts with other common macros. */
 #ifndef DISABLE_TLS_RESUME
   builtin_macro_create(US"_HAVE_TLS_RESUME");
 #endif
+#ifndef DISABLE_WELLKNOWN
+  builtin_macro_create(US"_HAVE_WELLKNOWN");
+#endif
+#ifdef EXPERIMENTAL_XCLIENT
+  builtin_macro_create(US"_HAVE_XCLIENT");
+#endif
+
+#ifdef USE_SQLITE
+  builtin_macro_create(US"_HAVE_HINTS_SQLITE");
+#elif defined(USE_TDB)
+  builtin_macro_create(US"_HAVE_HINTS_TDB");
+#elif defined(USE_DB)
+  builtin_macro_create(US"_HAVE_HINTS_BDB");
+#elif defined(USE_GDBM)
+  builtin_macro_create(US"_HAVE_HINTS_GDBM");
+#else
+  builtin_macro_create(US"_HAVE_HINTS_NDBM");
+#endif
 
 #ifdef LOOKUP_LSEARCH
   builtin_macro_create(US"_HAVE_LOOKUP_LSEARCH");
@@ -299,9 +327,6 @@ exp_features(void)
 #ifdef EXPERIMENTAL_DSN_INFO
   builtin_macro_create(US"_EXP_DSNI");
 #endif
-#ifdef EXPERIMENTAL_ESMTP_LIMITS
-  builtin_macro_create(US"_EXP_LIMITS");
-#endif
 #ifdef EXPERIMENTAL_QUEUEFILE
   builtin_macro_create(US"_EXP_QUEUEFILE");
 #endif
@@ -337,6 +362,7 @@ printf("#include \"exim.h\"\n");
 features();
 exp_features();
 options();
+expansions();
 params();
 
 printf("macro_item * macros = &p%d;\n", mp_index-1);
