@@ -3,7 +3,9 @@
 *************************************************/
 
 /* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 
 #include "../exim.h"
@@ -148,7 +150,9 @@ for (host_item * prev = NULL, * h = addr->host_list, *next_h; h; h = next_h)
 	if (lookup_type & LK_DEFAULT)
 	  {
 	  DEBUG(D_route|D_host_lookup)
-	    debug_printf("DNS lookup failed: trying getipnodebyname\n");
+	    debug_printf("DNS lookup failed: trying %s\n",
+	      f.running_in_test_harness
+	      ? "host_fake_gethostbyname" : "getipnodebyname");
 	  rc = host_find_byname(h, ignore_target_hosts, HOST_FIND_QUALIFY_SINGLE,
 	    &canonical_name, TRUE);
 	  }
